# flake8: noqa: F401
# Import tool modules into the aider.tools namespace

# Import all tool modules
from . import (
    command,
    command_interactive,
    context_manager,
    delete_block,
    delete_line,
    delete_lines,
    extract_lines,
    finished,
    git_branch,
    git_diff,
    git_log,
    git_remote,
    git_show,
    git_status,
    grep,
    indent_lines,
    insert_block,
    list_changes,
    ls,
    replace_all,
    replace_line,
    replace_lines,
    replace_text,
    show_numbered_context,
    thinking,
    undo_change,
    update_todo_list,
    view_files_matching,
    view_files_with_symbol,
)

# List of all available tool modules for dynamic discovery
TOOL_MODULES = [
    command,
    command_interactive,
    context_manager,
    delete_block,
    delete_line,
    delete_lines,
    extract_lines,
    finished,
    git_branch,
    git_diff,
    git_log,
    git_remote,
    git_show,
    git_status,
    grep,
    indent_lines,
    insert_block,
    list_changes,
    ls,
    replace_all,
    replace_line,
    replace_lines,
    replace_text,
    show_numbered_context,
    thinking,
    undo_change,
    update_todo_list,
    view_files_matching,
    view_files_with_symbol,
]
